###
### Copyright (c) 1999 The University of Utah and the Flux Group.
### All rights reserved.
###
### This file is part of Janos.
### 
### Janos is free software; you can redistribute it and/or modify it
### under the terms of the GNU General Public License, version 2, as
### published by the Free Software Foundation.
### 
### Janos is distributed in the hope that it will be useful, but
### WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details, in the file
### COPYING.  If that file is not present, write to the Free
### Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
### 
### The University of Utah and the Flux Group requests users to
### return to flux-dist@cs.utah.edu any improvements that they make.
###

###
### Define a package for compilation
###
### Expects 'JAVA_SOURCES' to be defined
### Expects 'PACKAGE_NAME' to be defined
###
### This makefile fragment should be included for each package
### it adds rules for preprocessing the package and compiling
### all of its sources.
###

# Package-specific directory names
$(PACKAGE_NAME)_FULLNAME:=$(PACKAGE_PREFIX).$(PACKAGE_NAME)
$(PACKAGE_NAME)_DEST_DIR:=$(subst .,/,$($(PACKAGE_NAME)_FULLNAME))
$(PACKAGE_NAME)_DIR_PREFIX:=$(subst .,/,$(PACKAGE_PREFIX))

# Save the set of sources for this package
$(PACKAGE_NAME)_SOURCES:=$(JAVA_SOURCES)
$(PACKAGE_NAME)_CPPSOURCES:=$(addprefix classes-$(LIBSTYLE)/$($(PACKAGE_NAME)_DIR_PREFIX)/, $(JAVA_SOURCES:.Java=.java))

# Reset the JAVA_SOURCES variable for the next package to use cleanly
JAVA_SOURCES:=

# Add the sources to the global list of all sources
ALL_SOURCES+=$($(PACKAGE_NAME)_SOURCES)
ALL_CPPSOURCES+=$($(PACKAGE_NAME)_CPPSOURCES)

# Post-processed source should be left around
.PRECIOUS: $($(PACKAGE_NAME)_CPPSOURCES)

## Generate a post-processed (.java) file from a .Java
## Depends on the file that defines the macros.
$(DEST_DIR)/$($(PACKAGE_NAME)_DIR_PREFIX)/%.java: %.Java $(GLOBAL_MACROS)
	@set -e 
	@echo Preprocessing $<
	@$(RM) $@
	@/bin/echo -n "/* !!! WARNING: this file was automatically generated from $< DO NOT EDIT !!! */" > $@
	@$(PREPROCESS) $< | $(DOC_MAGIC_SUBST) >> $@ || ($(RM) $@; $(FAILURE))

## Compile the package.  Sends all of the .java files to the compiler
## all at once.  Depends on the .java files all being up to date.
## The .compile_FOO file is actually created.  If the compile fails,
## The output is massaged to be in terms of the original .Java files (heh).
.compile_$(PACKAGE_NAME): $($(PACKAGE_NAME)_CPPSOURCES)
	@echo "Compiling $(subst .compile_,,$@) package"
	@rm -f $@ $(@)-output
	@if $(JAVAC) $^ > $(@)-output; then \
		touch $@;					\
	else 							\
		echo "FAILURE";					\
		sed -e 's/\.java/\.Java/' -e 's,$(DEST_DIR)/$(PACKAGE_PREFIX_DIR)/,,' $(@)-output;	\
		$(FAILURE); \
	fi


$(PACKAGE_NAME)_EXPECTED_CPP:=$(sort $($(PACKAGE_NAME)_CPPSOURCES))
$(PACKAGE_NAME)_ACTUAL_CPP_GLOB:=$(DEST_DIR)/$($(PACKAGE_NAME)_DEST_DIR)/*.java
$(PACKAGE_NAME)_ACTUAL_CPP:=$(sort $(wildcard $($(PACKAGE_NAME)_ACTUAL_CPP_GLOB)))
$(PACKAGE_NAME)_EXTRA_CPP=$(strip $(filter-out  $($(PACKAGE_NAME)_EXPECTED_CPP),  $($(PACKAGE_NAME)_ACTUAL_CPP)))

.PHONY: sourcecheck-$(PACKAGE_NAME)

sourcecheck-$(PACKAGE_NAME): $($(PACKAGE_NAME)_EXTRA_CPP)
	@if test ! -z "$^"; then \
	  echo "WARNING:  Package $(subst sourcecheck-,,$@) contains extra files: $(subst $(DEST_DIR)/,,$^)"; \
	fi

## Generate the package target directory if necesary
$(DEST_DIR)/$($(PACKAGE_NAME)_DEST_DIR):
	$(MKDIR) $@


## Add the directory to the list of required directories
DEST_DIRECTORIES+=$(DEST_DIR)/$($(PACKAGE_NAME)_DEST_DIR)

## Add the compile target to the list of package compile targets
COMPILATION_TARGETS+=.compile_$(PACKAGE_NAME) sourcecheck-$(PACKAGE_NAME)

### Cleanup target for this package
clean:: clean_$(PACKAGE_NAME)

clean_%::
	@echo Cleaning $*
	$(RM) $(DEST_DIR)/$($(*)_DEST_DIR)/*.java
	$(RM) $(DEST_DIR)/$($(*)_DEST_DIR)/*.class

### Debugging target for this package

debug:: debug_$(PACKAGE_NAME)

debug_%::
	@echo 
	@echo "**** Targets for $*"
	@echo "    $* Fullname    = $($(*)_FULLNAME)"
	@echo "    $* Dest Dir    = $($(*)_DEST_DIR)"
	@echo "    $* Prefix      = $($(*)_DIR_PREFIX)"
	@echo "    $* sources     = $($(*)_SOURCES)"
	@echo "    $* CPP sources = $($(*)_CPPSOURCES)"
	@echo "    $* EXPECTED_CPP sources = $($(*)_EXPECTED_CPP)"
	@echo "    $*   ACTUAL_CPP sources = $($(*)_ACTUAL_CPP)"
	@echo "    $*    EXTRA_CPP sources = $($(*)_EXTRA_CPP)"
	@echo "    $* GLOB        = $($(PACKAGE_NAME)_ACTUAL_CPP_GLOB)"
