/*
 * AppBooster.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTS_ANTS_PKG;

import JANOS_CORE_PKG.Flow;
import JANOS_CORE_PKG.ThreadPool;
import JANOS_CORE_PKG.CommSpaceHandle;
import JANOS_CORE_PKG.CommSpaceElement;

import JANOS_CORE_PKG.NullElementException;
import JANOS_CORE_PKG.InvalidTypeException;

import java.util.Vector;

IMPORT_DEBUG

/**
 * Cannot be made public.
 * XXX: well, at least it can't be instantiated by the unwashed masses.
 *
 * CommSpaceHandle should point to a valid CSH for
 * the Booster class, plus:
 *   [2] this will write the App port to this offset.
 */
public class AppBooster
	extends Booster
{
	/**
	 * Must have the appropriate key to create an AppBooster.
	 * Only the core can actually create Keys, so only trusted
	 * code can create AppBoosters...
	 *
	 * @exception SecurityException if the key is inappropriate
	 * @exception NullPointerException if the key is null
	 */
	public AppBooster(JANOS_CORE_PKG.ClassInstantiationKey key)
		throws SecurityException
	{
		key.checkEquals(AppBooster.class);
	}

	protected void run(Flow flow,
			   ThreadPool threadPool,
			   String[] args,
			   CommSpaceHandle commSpaceHandle)
		throws Exception
	{
		DEBUGONLY(flow.setName("Flow for App " +args[0]));

		Class appClass = Class.forName(args[0]);
		Application app = (Application) (appClass.newInstance());
		app.setName(args[0]);
		app.setArgs(args, 1);

		// setupLocalNode was inherited from Booster
		Node localNode = setupLocalNode(flow, commSpaceHandle);

		app.setNode(localNode);

		// Write the app port number where the Primordial Node expects it
//		commSpaceHandle.write(2, new CommSpaceElement(app.getPort()));
		localNode.attachApplication(app, app.getPort());

		// Fire off the app's entry point.
		app.start();
	}

}
