/* 
 *  $Id: BuiltinProtocol.Java,v 1.4 1999/09/07 21:15:53 tullmann Exp $
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */
package JANOS_ANTS_ANTS_PKG;

import JANOS_ANTS_UTILS_PKG.MD5;

import JANOS_CORE_PKG.DemultiplexKey;
import JANOS_CORE_PKG.DeliverFunc;
import JANOS_CORE_PKG.BufferHandle;

IMPORT_DEBUG

/**
 * Base class for all builtin protocols.  Builtin protocols are 
 * not demand loaded and are known by "well-defined" names.
 * <p>
 * Package access only.
 *
 * @author David Wetherall
 * %AUTHOR_PAT%
 */
PACKAGE class BuiltinProtocol
	extends Protocol
{
	/**
	 * Override Protocol's builtin to flag this class as a builtin.
	 */
	PACKAGE boolean builtin()
	{
		return true;
	}	
	
	/**
	 * Builtin protocols must use this version of addCapsule.
	 * This versions uses the given version string to generate
	 * the MD5 hash for the code ID.   (Thus, the version of
	 * a builtin protocol is not tied to the actual code.)
	 * <p>
	 * Each builtin protocol class has a version string associated
	 * with it.  See, for example, DataCapsule.
	 *
	 * @see DataCapsule#VERSION
	 */
	protected void addCapsule(String name, String version)
		throws Exception  // XXX
	{
		/*
		 * This is how the BuiltinProtocol.addCapsule() method
		 * differs from the Protocol version: compute codeID
		 * from the version string..
		 */
		MD5 id = new MD5();
		id.update(version);
		TypeID codeID = TypeID.wrap(id.hash()).intern();

		// Note that we don't bother loading the code for builtin
		// classes because it will (1) never be shipped anywhere
		// and (2) always loaded off the local disk before our
		// ClassLoader gets a chance to try it.

		CodeGroupData g = (CodeGroupData)groups.lastElement();
		g.classes.addElement(new CapsuleClassData(name, null, codeID));
	}
	
	/**
	 * Builtin protocols should never call this version of addCapsule.
	 *
	 * @see #addCapsule(java.lang.String,java.lang.String)
	 */
	protected void addCapsule(String name)
		throws Exception // XXX
	{
		panic("regular addCapsule() called on a builtin protocol!");
	}
}
