/*
 * CodeRequestDemultiplexKey.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTS_ANTS_PKG;

IMPORT_DEBUG

/**
 * Match ANTS packets for a specific ANTS protocol plus an
 * additional MID after the standard ANTS header.  Used to
 * match MIDIDRequest, DLRequest, and DLResponse capsules.
 */
PACKAGE class CodeRequestDemultiplexKey
	extends ANTSDemultiplexKey
{
	private final byte[] mid2Bytes;

	// ANTSDemultiplexKey will match mid1 (the MID from Capsule.Java),
	// and then we have to skip mid2Skip bytes to get to mid2
	// (the MID from MIDIDRequest/DLRequest/DLResponse):
	private static final int mid2Skip =
		Capsule.HEADERLENGTH - Xdr.TYPEID;

	PACKAGE CodeRequestDemultiplexKey(byte[] mid1Bytes, byte[] mid2Bytes)
	{
		super(mid1Bytes);
		assert(mid1Bytes);
		assert(mid1Bytes.length == 16);
		assert(mid2Bytes);
		assert(mid2Bytes.length == 16);
		this.mid2Bytes = mid2Bytes;
	}

	public int matches(byte[] buf, int packetOffset, int packetLength)
	{
		int antsMatch = super.matches(buf, packetOffset, packetLength);

		if (antsMatch == 0)
			return 0;

		return compareSlice(buf, packetOffset, packetLength,
				    antsMatch + mid2Skip, mid2Bytes);
	}

	public int whereDoesTheCapsuleDecodingActuallyStart(int payloadStart)
	{
		int rc = payloadStart - mid2Skip - mid2Bytes.length;
		assert(payloadStart > 0);
		assert(rc > 0); // always ANTS header at a minimum...
		return rc;
	}

	public String toString()
	{
		StringBuffer buf = new StringBuffer(50);

		buf.append(super.toString());
		buf.append("...[");
		assert(mid2Bytes != null);
		assert(mid2Bytes.length == 16);
		for (int i = 0; i < mid2Bytes.length; i++)
			buf.append(Integer.toHexString((mid2Bytes[i])&0xFF));
		buf.append("]");

		return buf.toString();
	}
}
