/*
 * AddressFactory.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_CORE_PKG;

/**
 * The Factory for creating Address objects.  Only one
 * Address factory is created, and it is created by
 * Address.getAddressFactory().  This is intended to allow
 * the Routing Table code (which is semi-trusted) to
 * be outside of the Core.
 *
 */
public class AddressFactory
{
	/**
	 * Package-private constructor.  Only invoked
	 * in Address.getAddressFactory().
	 *
	 * @see Address.getAddressFactory
	 */
	PACKAGE AddressFactory()
	{
	}

	/**
	 * Construct a new Address object for the given internet
	 * host and port.
	 */
	public Address newAddress(int anAddress, String iNetHostAddr, int iNetPort)
		throws NoSuchNodeAddress
	{
		java.net.InetAddress ia;
		
		try
		{
			ia = java.net.InetAddress.getByName(iNetHostAddr);
		}
		catch (java.net.UnknownHostException uhe)
		{
			throw new NoSuchNodeAddress(uhe.getMessage());
		}
		
		return new Address(anAddress, ia, iNetPort);
	}
}
