/*
 * ClassInstantiationKey.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_CORE_PKG;

/**
 * Blaouaufngg.  Janos uses reflection to instantiate the first object
 * in a flow, so that the object can't have pointers to objects in
 * other flows.  Unfortunately, the object's class usually belongs to
 * another package, so it must be public and have a public constructor
 * in order for JDK1.1-style reflection to work.  But this means anyone
 * can instantiate the class, which may not be desirable.  To prevent
 * this, Janos passes a ClassInstantiationKey into the constructor (if
 * there is a constructor taking one ClassInstantiationKey as an argument),
 * and the constructor verifies that the key matches its own class:
 *
 * public class Foo
 * {
 *     public Foo(ClassInstantiationKey key) throws SecurityException
 *     {
 *         key.checkEquals(Foo.class);
 *     }
 * }
 *
 */
public final class ClassInstantiationKey
{
	private final Class c1;

	PACKAGE ClassInstantiationKey(Class c1) {this.c1 = c1;}

	public void checkEquals(Class c2)
		throws SecurityException
	{
		if(c1 != c2) throw new SecurityException(
			c1 + " does not equal " + c2);
	}
}
