/*
 * CycleTimer.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_CORE_PKG;

#undef HAVE_NATIVE_CYCLETIMER // XXX

/**
 * Access to the underlying cycle timer.
 * <p>
 * XXX Make this more generic: a simple timer interface.
 *
 * %AUTHOR_CKH%
 */
public class CycleTimer
{
	public String name;
	public long count;
	public long time;
	public long timeSquareSum;

	public long start_time;

	public CycleTimer()
	{
	}

	public CycleTimer(String name)
	{
		this.name = name;
	}

	public synchronized void reset()
	{
		start_time = 0;
		count = 0;
		time = 0;
		timeSquareSum = 0;
	}

	public synchronized void start()
	{
		start_time = cycles();
	}

	public synchronized void end()
	{
		if(start_time != 0)
		{
			long dt = cycles() - start_time;
			time += dt;
			timeSquareSum += dt * dt;
			count++;
		}
	}

	public synchronized long avg()
	{
		return time / count;
	}

	public synchronized long stdDev()
	{
		double avg = time / count;
		return (long) Math.sqrt(
			((double) timeSquareSum) / count - avg * avg);
	}

#ifdef HAVE_NATIVE_CYCLETIMER
	public static native long cycles();

	static
	{
		System.loadLibrary("CycleTimer");
	}
#else
	public static long cycles() {return 0;}
#endif

}
