/*
 * OutChannel.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_CORE_PKG;

import java.io.IOException;

import java.net.DatagramSocket;
import java.net.DatagramPacket;

#ifdef BENCH
import JANOS_LIB_PKG.CycleTimer;
#endif

/**
 * Java wrapper for a Channel.
 * <p>
 * Implemented on top of UDP for the moment.
 */
public final class OutChannel
{
	PACKAGE final Flow flow;

	/**
	 */
	private final DatagramPacket sendPacket;

	/**
	 */
	private final byte[] gatherBuffer;

	/**
	 */
	private final OffsetLength offsetLength;

	/**
	 */
	private final DatagramSocket sendSocket;

#ifdef BENCH
	public static CycleTimer udpSendTimer = new CycleTimer("OutChannel udp send");
#endif //BENCH

	public OutChannel(Flow flow,
			  Address sendAddr)
		throws IOException
	{
		if(flow == null || sendAddr == null)
			throw new NullPointerException();

		this.flow = flow;

		gatherBuffer = new byte[UdpBuffer.MAX_PACKET_LEN];
		offsetLength = new OffsetLength();
		sendPacket = new DatagramPacket(gatherBuffer, 0,
						sendAddr.iNetAddr(), sendAddr.iNetPort());
		sendSocket = new DatagramSocket();
	}

	public synchronized void send(BufferHandle packet)
		throws IOException
	{
		final Buffer buf = packet.getBuffer(offsetLength);

		if(packet.getNext() == packet && offsetLength.offset == 0)
		{
			// Only one element in the aggregate:
			sendPacket.setData(buf.data);
			sendPacket.setLength(offsetLength.length);
		}
		else
		{
			synchronized(flow.state.lock)
			{
				// XXX move this to sendAggregate().

				// Multiple element aggregate: gather the data
				int len = 0;
				BufferHandle i = packet;
				do
				{
					int ilen = i.getLength();
					i.getBytes(0, gatherBuffer, len, ilen);
					len += ilen;
					i = i.getNext();
				} while(i != packet);

				sendPacket.setData(gatherBuffer);
				sendPacket.setLength(len);
			}
		}
		TRACE(Trace.CORE_OUTGOING, "sending: dest_port = " +sendPacket.getPort()+ " data = " +Debug.dbgPacketSummary(sendPacket.getData(), 0, sendPacket.getLength()));

		BENCHONLY(InChannel.recvToSendTimer.end());
		BENCHONLY(InChannel.sendToRecvTimer.start());
		BENCHONLY(udpSendTimer.start());
#ifndef BENCH
		sendSocket.send(sendPacket);
#else //BENCH
		InChannel.cheatLen = sendPacket.getLength();
		System.arraycopy(sendPacket.getData(), 0, InChannel.cheatBuffer, 0, InChannel.cheatLen);
		InChannel.cheatLock.signal();
#endif //BENCH
		BENCHONLY(udpSendTimer.end());

		buf.release();
	}
}


