/*
 * DebugControl.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_CORE_PKG;

import java.util.StringTokenizer;  // XXX

/**
 * Control for the trace macros spread through-out creation.
 *
 * %AUTHOR_PAT%
 */
public class Trace
{
	private static final String defaultFlags = "NODE_SETUP";
	private static final String flagPropKey  = "janos.debugFlags";

	private static final int MAX_FLAGS = 64; // # bits in long
	private static final boolean DEFAULT_VAL = true;

	private static final String[] names = new String[MAX_FLAGS];
	private static long flags;

	static 
	{
		String s;
		s = java.lang.System.getProperty(flagPropKey, defaultFlags);
		setFlags(s);
	}

	private static void setFlags(String flagsStr)
	{
		StringTokenizer st = new StringTokenizer(flagsStr,",|;");
	TOKENIZING:
		while(st.hasMoreTokens())
		{
			String flag = st.nextToken().intern();

			for (int i = 0; i < MAX_FLAGS; i++)
			{
				if (flag == names[i]) // everything is intern'd
				{
					setFlag(i);
					continue TOKENIZING;
				}
			}
		}
	}

	public static final boolean check(long mask) {
		return ((flags & mask) != 0);
	}

	private static final void setFlag(int idx) {
		flags |= (1<<idx);
	}

// Define a flag
#define FLAG(NAME, IDX)						\
	static {						\
		names[IDX] = #NAME.intern();			\
	}							\
	public static final long NAME = (1<<IDX)
		
		// There are only 64 flags available...
	FLAG(CORE_INCOMING, 1);
	FLAG(CORE_BUFH,     2);
	FLAG(CORE_OUTGOING, 3);
	FLAG(CORE_DISPATCH, 4);

	FLAG(ROUTE_READ_TABLE, 10);

	FLAG(ANTS_DEMULTIPLEX,    32);
	FLAG(ANTS_APP,            33);
	FLAG(ANTS_CAPSULECHANNEL, 34);
	FLAG(ANTS_STARTUP,        35);
	FLAG(ANTS_DL,             36);
	FLAG(ANTS_OUTGOING,       37);
	FLAG(ANTS_ROUTING,        38);
}
