#!/usr/local/bin/perl -w
#
# Dump a list of all the classes referenced from the given .class
# file.
#

#
# Copyright (c) 1999 University of Utah and the Flux Group.
# All rights reserved.
#
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
# 
# Written by Patrick Tullmann <tullmann@cs.utah.edu>
#

use JavaClass;

## Parse the command line
my $classFile = shift  || &usage();

## Read/parse the class file
my $class = &JavaClass::readClass($classFile);

## Print the referenced classes Look through the constant pool for
## 'field' and 'method' types.  For each, add the associated class,
## and parse the descriptor from the associated NameAndType.
##
## Add the superclass and all referenced interface classes.
##
## Put them all in a hashtable and print the resulting table.
##

my %classes = ();
my %class = %{$class};

$i = 1;
while ($i < $class{constantPoolCt}) {
  if (! defined($class{constantPool}[$i])) {
    print STDERR "[Constant pool entry $i is invalid.]\n";
    $i++;
    next;
  }

  my %cpEntry = %{$class{constantPool}[$i]};

  if (($cpEntry{tag} eq $JavaClass::CONSTANT_FieldRef) 
      || ($cpEntry{tag} eq $JavaClass::CONSTANT_MethodRef)) {
    my $classIndex = $cpEntry{classIndex};
    my $nameTypeIndex = $cpEntry{nameTypeIndex};

    ## Get the class name
    %cpEntry = %{$class{constantPool}[$classIndex]};
    my $ni = $cpEntry{nameIndex};
    %cpEntry = %{$class{constantPool}[$ni]};
    $classes{$cpEntry{val}} = '1'; ## XXX increment counter

    ## Follow NameAndType to descriptor to the UTF8 
    %cpEntry = %{$class{constantPool}[$nameTypeIndex]};
    my $di = $cpEntry{descriptorIndex};
    %cpEntry = %{$class{constantPool}[$di]};
    my $typeDescriptorStr = $cpEntry{val};

    ## Pull all object references from the stream:
    #print STDERR ":: $typeDescriptorStr :: \n";
    while ($typeDescriptorStr =~ /L([^;]+)\;/g) {
      #print STDERR "::    $1\n";
      $classes{$1} = $1;
    }
  }

  ## Check next entry
  $i++;
}

## Add the superclass
my $superClassNameIndex = %{$class{constantPool}[$class{superClass}]}->{nameIndex};
my $clname = %{$class{constantPool}[$superClassNameIndex]}->{val};
$classes{$clname} = '1';

## Add interface classes
if ($class{interfaceCt} > 0) {
  $i = 0;
  while ($i < $class{interfaceCt}) {
    my $iClass = $class{interfaces}[$i];
    my %iClassConst = %{$class{constantPool}[$iClass]};
    my $iClassName = $iClassConst{nameIndex};
    my $interfaceName = %{$class{constantPool}[$iClassName]}->{val};
    #print "\t$i] @ $iClass ($interfaceName)\n";
    $classes{$interfaceName} = '1';
    
    $i++;
  }

}

## Print the array
print "$classFile references: \n";
foreach $i (sort(keys(%classes)))  {
  print " $i\n";
}


###
###
###

sub usage() {
  print STDOUT "Usage:\n";
  print STDOUT "\tclassRefs.pl <ClassFile>\n";
  print STDOUT "\t\tNote that '.class' will be appended if it is not specified in the file name.\n";

  exit 11;
}

#eof
